package com.nokia.finance.tasks.dao.car;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.LinkedHashMap;
import java.util.List;

@Mapper
public interface CarStrategyDao {
    /**
     * 插入高油耗车辆策略
     *
     * @param startYearMonth 开始年月
     * @param endYearMonth 结束年月
     */
    @Update("""
with
t101 as (
    select area_no,
        area_name,
        city_no,
        city_name,
        sum(ran_you) as ran_you_sum
    from car.car_fei_yong_month
    where first_unit != '省公司本部'
        and position('建设部' in che_liang_suo_shu_dan_wei) = 0
        and area_no is not null
        and area_no != ''
        and city_no is not null
        and city_no != ''
        and year_month >= #{startYearMonth}
        and year_month <= #{endYearMonth}
    group by area_no,
        area_name,
        city_no,
        city_name
),
t102 as (
    select area_no,
        area_name,
        city_no,
        city_name,
        sum(zong_li_cheng) as zong_li_cheng_sum
    from car.car_li_cheng_month
    where first_unit != '省公司本部'
        and position('建设部' in che_liang_suo_shu_dan_wei) = 0
        and area_no is not null
        and area_no != ''
        and city_no is not null
        and city_no != ''
        and year_month >= #{startYearMonth}
        and year_month <= #{endYearMonth}
    group by area_no,
        area_name,
        city_no,
        city_name
),
t103 as (
select
t101.*,
t102.zong_li_cheng_sum
from t101
join t102 on t101.area_no = t102.area_no and t101.city_no = t102.city_no
),
t104 as (
    select *,
        case when zong_li_cheng_sum = 0 then null else ran_you_sum / zong_li_cheng_sum * 100 end as bai_gong_li_you_fei
    from t103
    where ran_you_sum > 0 and zong_li_cheng_sum > 0
),
t105 as (
    select *,
        row_number() over (
            order by bai_gong_li_you_fei desc
        ) as bai_gong_li_you_fei_rank
    from t104
),
t111 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_010_1' as kpi_code,
        '县公司车辆燃油费' as kpi_name,
        round(ran_you_sum, 2) as kpi_value,
        bai_gong_li_you_fei_rank as sort
    from t105
),
t112 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_010_2' as kpi_code,
        '车辆行驶里程' as kpi_name,
        round(zong_li_cheng_sum, 2) as kpi_value,
        bai_gong_li_you_fei_rank as sort
    from t105
),
t113 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_010_3' as kpi_code,
        '百公里油耗' as kpi_name,
        round(bai_gong_li_you_fei, 2) as kpi_value,
        bai_gong_li_you_fei_rank as sort
    from t105
),
t114 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_010_4' as kpi_code,
        '排名' as kpi_name,
        bai_gong_li_you_fei_rank as kpi_value,
        bai_gong_li_you_fei_rank as sort
    from t105
),
t121 as (
    select * from t111
    union all
    select * from t112
    union all
    select * from t113
    union all
    select * from t114
)
insert into publish.car_high_fuel_consumption(
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1,
sort
)
select
#{endYearMonth} as acct_date,
'30059' as dept_code,
'河北省分公司财务部' as dept_name,
'30059_010' as strategy_code,
area_no,
area_name,
city_no,
city_name,
'' as sale_no,
'' as sale_name,
kpi_code,
kpi_name,
kpi_value,
'' as kpi_type,
'' as remark_1,
sort
from t121
order by sort,
    kpi_code
""")
    int insertCarHighFuelConsumption(@Param("startYearMonth") Integer startYearMonth,
                                     @Param("endYearMonth") Integer endYearMonth);

    /**
     * 查询高油耗车辆策略
     *
     * @param endYearMonth 账期
     */
    @Select("""
select
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
from publish.car_high_fuel_consumption where acct_date = #{endYearMonth} order by sort, kpi_code
""")
    List<LinkedHashMap<String, Object>> getCarHighFuelConsumption(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入高修理车辆策略
     * @param startYearMonth 开始年月
     * @param endYearMonth 结束年月
     */
    @Update("""
with
t101 as (
    select area_no,
        area_name,
        city_no,
        city_name,
        sum(wei_xiu) as wei_xiu_sum
    from car.car_fei_yong_month
    where first_unit != '省公司本部'
        and position('建设部' in che_liang_suo_shu_dan_wei) = 0
        and che_liang_lai_yuan = '自有车辆'
        and area_no is not null
        and area_no != ''
        and city_no is not null
        and city_no != ''
        and year_month >= #{startYearMonth}
        and year_month <= #{endYearMonth}
    group by area_no,
        area_name,
        city_no,
        city_name
),
t102 as (
    select area_no,
        area_name,
        city_no,
        city_name,
        count(1) as total
    from car.car_base_data_month
    where first_unit != '省公司本部'
        and position('建设部' in che_liang_suo_shu_dan_wei) = 0
        and che_liang_lai_yuan = '自有车辆'
        and area_no is not null
        and area_no != ''
        and city_no is not null
        and city_no != ''
        and year_month = #{endYearMonth}
    group by area_no,
        area_name,
        city_no,
        city_name
),
t103 as (
select
t101.*,
t102.total
from t101
join t102 on t101.area_no = t102.area_no and t101.city_no = t102.city_no
),
t104 as (
    select t103.*,
        case when wei_xiu_sum is null then 0 else wei_xiu_sum / total end as wei_xiu_avg
    from t103
    where wei_xiu_sum > 0
),
t105 as (
    select *,
        row_number() over (
            order by wei_xiu_avg desc
        ) as wei_xiu_rank
    from t104
),
t111 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_011_1' as kpi_code,
        '自有车辆维修费' as kpi_name,
        round(wei_xiu_sum, 2) as kpi_value,
        wei_xiu_rank as sort
    from t105
),
t112 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_011_2' as kpi_code,
        '自有车辆数' as kpi_name,
        total as kpi_value,
        wei_xiu_rank as sort
    from t105
),
t113 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_011_3' as kpi_code,
        '车均维修费' as kpi_name,
        round(wei_xiu_avg, 2) as kpi_value,
        wei_xiu_rank as sort
    from t105
),
t114 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_011_4' as kpi_code,
        '排名' as kpi_name,
        wei_xiu_rank as kpi_value,
        wei_xiu_rank as sort
    from t105
),
t121 as (
    select * from t111
    union all
    select * from t112
    union all
    select * from t113
    union all
    select * from t114
)
insert into publish.car_high_repair(
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1,
sort
)
select
#{endYearMonth} as acct_date,
'30059' as dept_code,
'河北省分公司财务部' as dept_name,
'30059_011' as strategy_code,
area_no,
area_name,
city_no,
city_name,
'' as sale_no,
'' as sale_name,
kpi_code,
kpi_name,
kpi_value,
'' as kpi_type,
'' as remark_1,
sort
from t121
order by sort,
    kpi_code
""")
    int insertCarHighRepair(@Param("startYearMonth") Integer startYearMonth,
                            @Param("endYearMonth") Integer endYearMonth);

    /**
     * 查询高修理车辆策略
     *
     * @param endYearMonth 账期
     */
    @Select("""
select
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
from publish.car_high_repair where acct_date = #{endYearMonth} order by sort, kpi_code
""")
    List<LinkedHashMap<String, Object>> getCarHighRepair(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入租赁车辆产生维修费策略
     * @param startYearMonth 开始年月
     * @param endYearMonth 结束年月
     */
    @Update("""
with
t101 as (
    select *
    from car.car_base_data_month
    where che_liang_lai_yuan = '租赁车辆'
    and area_no is not null
    and area_no != ''
    and city_no is not null
    and city_no != ''
    and year_month = #{startYearMonth}
),
t102 as (
    select *
    from car.car_fei_yong_month
    where year_month >= #{startYearMonth}
        and year_month <= #{endYearMonth}
),
t103 as (
    select
        area_no,
        che_pai_hao,
        sum(wei_xiu) as wei_xiu_sum
    from t102
    group by che_pai_hao, area_no
),
t104 as (
select
t101.*,
t103.wei_xiu_sum
from t101 join t103 on t101.che_pai_hao = t103.che_pai_hao and t101.area_no = t103.area_no
where t103.wei_xiu_sum > 1800
),
t111 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_021_1' as kpi_code,
        '车辆个数' as kpi_name,
        count(1)::varchar as kpi_value
    from t104
    group by area_no, area_name, city_no, city_name
),
t112 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_021_2' as kpi_code,
        '车牌号' as kpi_name,
        che_pai_hao as kpi_value
    from t104
),
t121 as (
    select * from t111
    union all
    select * from t112
)
insert into publish.car_rental_repair(
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
)
select
#{endYearMonth} as acct_date,
'30059' as dept_code,
'河北省分公司财务部' as dept_name,
'30059_021' as strategy_code,
area_no,
area_name,
city_no,
city_name,
'' as sale_no,
'' as sale_name,
kpi_code,
kpi_name,
kpi_value,
'' as kpi_type,
'' as remark_1
from t121
order by
    area_no,
    area_name,
    city_no,
    city_name,
    kpi_code
""")
    int insertCarRentalRepair(@Param("startYearMonth") Integer startYearMonth,
                              @Param("endYearMonth") Integer endYearMonth);

    /**
     * 查询租赁车辆产生维修费策略
     *
     * @param endYearMonth 账期
     */
    @Select("""
select
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
from publish.car_rental_repair
where acct_date = #{endYearMonth}
order by
    area_no,
    area_name,
    city_no,
    city_name,
    kpi_code
""")
    List<LinkedHashMap<String, Object>> getCarRentalRepair(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 判断是否有租赁车辆产生维修费策略需要的数据
     * @param endYearMonth 账期
     */
    @Select("""
select
    (
    select
        exists (
        select
            1
        from
            car.car_base_data_month
        where
            year_month = #{endYearMonth}))
    and (
    select
        exists (
        select
            1
        from
            car.car_fei_yong_month
        where
            year_month = #{endYearMonth}))
""")
    boolean hasCarRentalRepair(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 判断是否有车辆违章未处理策略需要的数据
     * @param endYearMonth 账期
     */
    @Select("""
select exists (select 1 from car.car_wei_zhang where year_month = #{endYearMonth})
""")
    boolean hasCarViolationUnprocessed(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入车辆违章未处理策略
     * @param endYearMonth 结束年月
     */
    @Update("""
with
t101 as (
select
distinct on (che_pai_hao, area_no, city_no) *
from
    car.car_wei_zhang_chang_qi
where
    area_no is not null
    and area_no != ''
    and city_no is not null
    and city_no != ''
    and year_month = #{endYearMonth}
),
t111 as (
    select 
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_022_1' as kpi_code,
        '车辆个数' as kpi_name,
        count(1)::varchar as kpi_value
    from t101
    group by area_no, area_name, city_no, city_name
),
t112 as (
    select 
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_022_2' as kpi_code,
        '车牌号' as kpi_name,
        che_pai_hao as kpi_value
    from t101
),
t121 as (
    select * from t111
    union all
    select * from t112
)
insert into publish.car_violation_unprocessed(
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
)
select
#{endYearMonth} as acct_date,
'30059' as dept_code,
'河北省分公司财务部' as dept_name,
'30059_022' as strategy_code,
area_no,
area_name,
city_no,
city_name,
'' as sale_no,
'' as sale_name,
kpi_code,
kpi_name,
kpi_value,
'' as kpi_type,
'' as remark_1
from t121
order by
    area_no,
    area_name,
    city_no,
    city_name,
    kpi_code
""")
    int insertCarViolationUnprocessed(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 查询车辆违章未处理策略
     *
     * @param endYearMonth 账期
     */
    @Select("""
select
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
from publish.car_violation_unprocessed
where acct_date = #{endYearMonth}
order by
    area_no,
    area_name,
    city_no,
    city_name,
    kpi_code
""")
    List<LinkedHashMap<String, Object>> getCarViolationUnprocessed(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入违规车辆策略
     * @param endYearMonth 账期
     */
    @Update("""
with
t101 as (
select
    area_no,
    area_name,
    count(1) as total
from
    car.car_wei_gui
where
    area_no is not null
    and area_no != ''
    and city_no is not null
    and city_no != ''
    and year_month = #{endYearMonth}
group by
    area_name, area_no
),
t102 as (
select * from car.car_wei_gui
where
    area_no is not null
    and area_no != ''
    and city_no is not null
    and city_no != ''
    and year_month = #{endYearMonth}
),
t111 as (
    select
        area_no,
        area_name,
        'kpi_30059_012_1' as kpi_code,
        '违规次数' as kpi_name,
        total::varchar as kpi_value
    from t101
),
t112 as (
    select
        area_no,
        area_name,
        'kpi_30059_012_2' as kpi_code,
        '车牌号' as kpi_name,
        che_pai_hao as kpi_value
    from t102
),
t121 as (
    select * from t111
    union all
    select * from t112
)
insert into publish.car_wdyj(
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
)
select
#{endYearMonth} as acct_date,
'30059' as dept_code,
'河北省分公司财务部' as dept_name,
'30059_012' as strategy_code,
area_no,
area_name,
'' as city_no,
'' as city_name,
'' as sale_no,
'' as sale_name,
kpi_code,
kpi_name,
kpi_value,
'' as kpi_type,
'' as remark_1
from t121
order by
    area_no,
    area_name,
    kpi_code,
    kpi_value
""")
    int insertCarWdyj(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 查询违规车辆策略
     *
     * @param endYearMonth 账期
     */
    @Select("""
select
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
from publish.car_wdyj
where acct_date = #{endYearMonth}
order by
    area_no,
    area_name,
    kpi_code,
    kpi_value
""")
    List<LinkedHashMap<String, Object>> getCarWdyj(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 判断是否有低效车辆策略需要的数据
     * @param endYearMonth 账期
     */
    @Select("""
select
    (
    select
        exists (
        select
            1
        from
            car.car_base_data_month
        where
            year_month = #{endYearMonth}))
    and (
    select
        exists (
        select
            1
        from
            car.car_li_cheng_month
        where
            year_month = #{endYearMonth}))
""")
    boolean hasCarInefficiency(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入车辆低效占比策略
     * @param endYearMonth 账期
     */
    @Update("""
with
t101 as (
select * from car.car_di_xiao_month
where
    area_no is not null
    and area_no != ''
    and year_month = #{endYearMonth}
),
t102 as (
    select area_no,
        area_name,
        count(1) as di_xiao_count
    from t101
    group by area_no,
        area_name
),
t103 as (
    select area_no,
        area_name,
        count(1) as total
    from car.car_base_data_month
where
    area_no is not null
    and area_no != ''
    and year_month = #{endYearMonth}
    group by area_no,
        area_name
),
t104 as (
    select
    t102.*,
    t103.total,
    t102.di_xiao_count / t103.total::numeric * 100 as di_xiao_percent
    from t102
        join t103 on t102.area_no = t103.area_no
),
t111 as (
    select
        area_no,
        area_name,
        'kpi_30059_014_1' as kpi_code,
        '低效车个数' as kpi_name,
        di_xiao_count as kpi_value
    from t104
),
t112 as (
    select
        area_no,
        area_name,
        'kpi_30059_014_2' as kpi_code,
        '低效车占比' as kpi_name,
        round(di_xiao_percent, 2) as kpi_value
    from t104
    order by area_no
),
t121 as (
    select * from t111
    union all
    select * from t112
)
insert into publish.car_inefficiency_percent(
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
)
select
#{endYearMonth} as acct_date,
'30059' as dept_code,
'河北省分公司财务部' as dept_name,
'30059_014' as strategy_code,
area_no,
area_name,
'' as city_no,
'' as city_name,
'' as sale_no,
'' as sale_name,
kpi_code,
kpi_name,
kpi_value,
'' as kpi_type,
'' as remark_1
from t121
order by area_no,
    kpi_code
""")
    int insertCarInefficiencyPercent(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 查询车辆低效占比策略
     *
     * @param endYearMonth 账期
     */
    @Select("""
select
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
from publish.car_inefficiency_percent
where acct_date = #{endYearMonth}
order by
    area_no,
    kpi_code
""")
    List<LinkedHashMap<String, Object>> getCarInefficiencyPercent(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入车辆低效区县策略
     * @param endYearMonth 账期
     */
    @Update("""
with
t101 as (
select * from car.car_di_xiao_month
where
    area_no is not null
    and area_no != ''
    and city_no is not null
    and city_no != ''
    and year_month = #{endYearMonth}
),
t111 as (
    select 
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_013_1' as kpi_code,
        '低效车个数' as kpi_name,
        count(1)::varchar as kpi_value
    from t101
    group by area_no,
        area_name,
        city_no,
        city_name
),
t112 as (
    select
        area_no,
        area_name,
        city_no,
        city_name,
        'kpi_30059_013_2' as kpi_code,
        '车牌号' as kpi_name,
        che_pai_hao as kpi_value
    from t101
),
t121 as (
    select * from t111
    union all
    select * from t112
)
insert into publish.car_inefficiency_qx(
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
)
select
#{endYearMonth} as acct_date,
'30059' as dept_code,
'河北省分公司财务部' as dept_name,
'30059_013' as strategy_code,
area_no,
area_name,
city_no,
city_name,
'' as sale_no,
'' as sale_name,
kpi_code,
kpi_name,
kpi_value,
'' as kpi_type,
'' as remark_1
from t121
order by area_no,
    city_no,
    kpi_code
""")
    int insertCarInefficiencyQx(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 查询车辆低效占比策略
     *
     * @param endYearMonth 账期
     */
    @Select("""
select
acct_date,
dept_code,
dept_name,
strategy_code,
area_no,
area_name,
city_no,
city_name,
sale_no,
sale_name,
kpi_code,
kpi_name,
kpi_value,
kpi_type,
remark_1
from publish.car_inefficiency_qx
where acct_date = #{endYearMonth}
order by
    area_no,
    city_no,
    kpi_code
""")
    List<LinkedHashMap<String, Object>> getCarInefficiencyQx(@Param("endYearMonth") Integer endYearMonth);
}
