package com.nokia.finance.tasks.pojo.vo.car_report;

import com.nokia.finance.tasks.config.CarReportConfig;
import com.nokia.finance.tasks.pojo.po.car_report.CarWzPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWzUnitPo;
import lombok.Data;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Data
public class CarWzclycVo {
    /**
     * 违章长期未处理一级单位统计
     */
    private List<CarWzUnitVo> wzUnitList;

    public CarWzclycVo(List<CarWzUnitPo> wzUnitList, List<CarWzPo> wzList, CarReportConfig carReportConfig) throws IOException {
        this.wzUnitList = new ArrayList<>();
        for (CarWzUnitPo carWzUnitPo : wzUnitList) {
            this.wzUnitList.add(new CarWzUnitVo(carWzUnitPo));
        }
        writeWzXlsx(wzList, carReportConfig);
    }

    /**
     * 违章长期未处理xlsx
     */
    private void writeWzXlsx(List<CarWzPo> wzList, CarReportConfig carReportConfig) throws IOException {
        try (SXSSFWorkbook wb = new SXSSFWorkbook();
//             FastByteArrayOutputStream os = new FastByteArrayOutputStream()
             OutputStream os = Files.newOutputStream(Paths.get(carReportConfig.getWzXlsx()))
        ) {
            DataFormat dataFormat = wb.createDataFormat();
            // 数字样式
            CellStyle numberCellStyle = wb.createCellStyle();
            numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
            numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
            // 整数样式
            CellStyle integerCellStyle = wb.createCellStyle();
            integerCellStyle.setDataFormat(dataFormat.getFormat("#,##0"));
            integerCellStyle.setAlignment(HorizontalAlignment.RIGHT);
            // 日期时间样式
            CellStyle dateTimeCellStyle = wb.createCellStyle();
            dateTimeCellStyle.setDataFormat(dataFormat.getFormat("yyyy/MM/dd HH:mm:ss"));
            // 表头样式
            CellStyle headerCellStyle = wb.createCellStyle();
            headerCellStyle.setAlignment(HorizontalAlignment.CENTER);
            headerCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            SXSSFSheet sheet = wb.createSheet();
            AtomicInteger rowIndex = new AtomicInteger(0);
            // 表头
            Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
            List<String> headers = Stream.of("车牌号", "单位", "二级", "三级", "违章时间", "违章地点", "违章详情",
                    "扣分", "罚款", "违章未处理时长（天）", "线下实际处理状态（最终状态）").toList();
            int headerSize = headers.size();
            for (int i = 0; i < headerSize; i++) {
                Cell cell = headerRow.createCell(i);
                cell.setCellValue(headers.get(i));
                cell.setCellStyle(headerCellStyle);
                // 根据内容长度设置列宽
                int columnWidth = headers.get(i).length() * 256 * 2 + 256 * 12;
                sheet.setColumnWidth(i, columnWidth);
            }
            for (CarWzPo t : wzList) {
                AtomicInteger columnIndex = new AtomicInteger(0);
                Row row = sheet.createRow(rowIndex.getAndIncrement());
                // 车牌号
                Cell chePaiHaoCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getChePaiHao() != null) {
                    chePaiHaoCell.setCellValue(t.getChePaiHao());
                }
                // 单位
                Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getFirstUnit() != null) {
                    firstUnitCell.setCellValue(t.getFirstUnit());
                }
                // 二级
                Cell secondUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getSecondUnit() != null) {
                    secondUnitCell.setCellValue(t.getSecondUnit());
                }
                // 三级
                Cell thirdUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getThirdUnit() != null) {
                    thirdUnitCell.setCellValue(t.getThirdUnit());
                }
                // 违章时间
                Cell weiZhangShiJianCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getWeiZhangShiJian() != null) {
                    weiZhangShiJianCell.setCellValue(t.getWeiZhangShiJian());
                }
                weiZhangShiJianCell.setCellStyle(dateTimeCellStyle);
                // 违章地点
                Cell weiZhangDiDianCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getWeiZhangDiDian() != null) {
                    weiZhangDiDianCell.setCellValue(t.getWeiZhangDiDian());
                }
                // 违章详情
                Cell weiZhangXiangQingCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getWeiZhangXiangQing() != null) {
                    weiZhangXiangQingCell.setCellValue(t.getWeiZhangXiangQing());
                }
                // 扣分
                Cell kouFenCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getKouFen() != null) {
                    kouFenCell.setCellValue(t.getKouFen());
                }
                kouFenCell.setCellStyle(integerCellStyle);
                // 罚款
                Cell faKuanCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getFaKuan() != null) {
                    faKuanCell.setCellValue(t.getFaKuan().doubleValue());
                }
                faKuanCell.setCellStyle(integerCellStyle);
                // 违章未处理时长
                Cell weiZhangWeiChuLiShiChangCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getWeiZhangWeiChuLiShiChang() != null) {
                    weiZhangWeiChuLiShiChangCell.setCellValue(t.getWeiZhangWeiChuLiShiChang());
                }
                weiZhangWeiChuLiShiChangCell.setCellStyle(integerCellStyle);
                // 线下实际处理状态
                Cell chuLiZhuangTaiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getChuLiZhuangTai() != null) {
                    chuLiZhuangTaiCell.setCellValue(t.getChuLiZhuangTai());
                }
            }
            wb.write(os);
        }
    }
}
