package com.nokia.finance.tasks.pojo.vo.car_report;

import com.nokia.finance.tasks.config.CarReportConfig;
import com.nokia.finance.tasks.pojo.po.car_report.CarRyycPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarRyycUnitPo;
import lombok.Data;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Data
public class CarYcryfVo {
    /**
     * 燃油异常一级单位统计
     */
    private List<CarRyycUnitVo> ryycUnitList;

    public CarYcryfVo(List<CarRyycUnitPo> ryycUnitList, List<CarRyycPo> carRyycPoList, CarReportConfig carReportConfig)
            throws IOException {
        this.ryycUnitList = new ArrayList<>();
        for (CarRyycUnitPo carRyycUnitPo : ryycUnitList) {
            this.ryycUnitList.add(new CarRyycUnitVo(carRyycUnitPo));
        }
        writeRyycXlsx(carRyycPoList, carReportConfig);
    }

    /**
     * 燃油异常xlsx
     */
    private void writeRyycXlsx(List<CarRyycPo> carRyycPoList, CarReportConfig carReportConfig) throws IOException {
        try (SXSSFWorkbook wb = new SXSSFWorkbook();
//             FastByteArrayOutputStream os = new FastByteArrayOutputStream()
             OutputStream os = Files.newOutputStream(Paths.get(carReportConfig.getRyycXlsx()))
        ) {
            DataFormat dataFormat = wb.createDataFormat();
            // 数字样式
            CellStyle numberCellStyle = wb.createCellStyle();
            numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
            numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
            // 表头样式
            CellStyle headerCellStyle = wb.createCellStyle();
            headerCellStyle.setAlignment(HorizontalAlignment.CENTER);
            headerCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            SXSSFSheet sheet = wb.createSheet();
            AtomicInteger rowIndex = new AtomicInteger(0);
            // 表头
            Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
            List<String> headers = Stream.of("车牌号", "单位", "二级", "三级", "车辆所属单位", "车辆来源", "车辆类型",
                    "车辆使用性质", "累计燃油", "累计里程", "百公里油费", "百公里油耗", "排名").toList();
            int headerSize = headers.size();
            for (int i = 0; i < headerSize; i++) {
                Cell cell = headerRow.createCell(i);
                cell.setCellValue(headers.get(i));
                cell.setCellStyle(headerCellStyle);
                // 根据内容长度设置列宽
                int columnWidth = headers.get(i).length() * 256 * 2 + 256 * 5;
                sheet.setColumnWidth(i, columnWidth);
            }
            for (CarRyycPo t : carRyycPoList) {
                AtomicInteger columnIndex = new AtomicInteger(0);
                Row row = sheet.createRow(rowIndex.getAndIncrement());
                // 车牌号
                Cell chePaiHaoCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getChePaiHao() != null) {
                    chePaiHaoCell.setCellValue(t.getChePaiHao());
                }
                // 单位
                Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getFirstUnit() != null) {
                    firstUnitCell.setCellValue(t.getFirstUnit());
                }
                // 二级
                Cell secondUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getSecondUnit() != null) {
                    secondUnitCell.setCellValue(t.getSecondUnit());
                }
                // 三级
                Cell thirdUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getThirdUnit() != null) {
                    thirdUnitCell.setCellValue(t.getThirdUnit());
                }
                // 车辆所属单位
                Cell cheLiangSuoShuDanWeiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangSuoShuDanWei() != null) {
                    cheLiangSuoShuDanWeiCell.setCellValue(t.getCheLiangSuoShuDanWei());
                }
                // 车辆来源
                Cell cheLiangLaiYuanCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangLaiYuan() != null) {
                    cheLiangLaiYuanCell.setCellValue(t.getCheLiangLaiYuan());
                }
                // 车辆类型
                Cell cheLiangLeiXingCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangLeiXing() != null) {
                    cheLiangLeiXingCell.setCellValue(t.getCheLiangLeiXing());
                }
                // 车辆使用性质
                Cell cheLiangShiYongXingZhiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangShiYongXingZhi() != null) {
                    cheLiangShiYongXingZhiCell.setCellValue(t.getCheLiangShiYongXingZhi());
                }
                // 燃油
                Cell ranYouSumCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getRanYouSum() != null) {
                    ranYouSumCell.setCellValue(t.getRanYouSum().doubleValue());
                }
                ranYouSumCell.setCellStyle(numberCellStyle);
                // 里程
                Cell zongLiChengSumCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getZongLiChengSum() != null) {
                    zongLiChengSumCell.setCellValue(t.getZongLiChengSum().doubleValue());
                }
                zongLiChengSumCell.setCellStyle(numberCellStyle);
                // 百公里油费
                Cell baiGongLiYouFeiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getBaiGongLiYouFei() != null) {
                    baiGongLiYouFeiCell.setCellValue(t.getBaiGongLiYouFei().doubleValue());
                }
                baiGongLiYouFeiCell.setCellStyle(numberCellStyle);
                // 百公里油耗
                Cell baiGongLiYouHaoCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getBaiGongLiYouHao() != null) {
                    baiGongLiYouHaoCell.setCellValue(t.getBaiGongLiYouHao().doubleValue());
                }
                baiGongLiYouHaoCell.setCellStyle(numberCellStyle);
                // 排名
                Cell baiGongLiYouFeiRankCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getBaiGongLiYouFeiRank() != null) {
                    baiGongLiYouFeiRankCell.setCellValue(t.getBaiGongLiYouFeiRank().doubleValue());
                }
            }
            wb.write(os);
        }
    }
}
