package com.nokia.finance.tasks.pojo.vo.house_report;

import com.nokia.finance.tasks.pojo.po.house_report.HouseFangWuWeiXiuZongTiQingKuangPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseRentOutIncomeStatPo;
import lombok.Data;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Data
public class HouseFcczztqkVo {
    /**
     * 全省房产出租总收入
     */
    private String zsr;
    /**
     * 同比增减
     */
    private String czsrtbzj;
    /**
     * 预算完成率
     */
    private String yswcl;
    /**
     * 序时进度差距
     */
    private String xsjdcj;
    /**
     * 出租收入实现同比增长单位数量
     */
    private String czsrtbzzUnitCount;
    /**
     * 出租收入增加最多单位
     */
    private String czsrzjMaxUnit;
    /**
     * 出租收入同比下降单位数量
     */
    private String czsrtbxjUnitCount;
    /**
     * 出租收入同比下降较大单位
     */
    private String czsrtbxjjdUnit;
    /**
     * 出租收入预算完成率低于序时进度，且收入同比下降较大单位
     */
    private String czsrdyxsjdtbxjjdUnit;
    /**
     * 出租收入表格
     */
    private List<HouseCzsrTableVo> czsrTable;
    /**
     * 出租欠费表格
     */
    private List<HouseCzqfTableVo> czqfTable;
    /**
     * 存在较大金额欠费单位
     */
    private String czqfjdUnit;
    /**
     * 出租收入同比增长幅度加大，但存在较大金额欠费单位
     */
    private String czsrtbzzqfjdUnit;
    /**
     * 出租收入同比增长幅度加大，但存在较大金额欠费
     */
    private String czsrtbzzqfjd;
    /**
     * 出租收入预算完成率低于序时进度，同比下降，且累计欠费
     */
    private String czsrdyxsjdtbxjqf;

    public HouseFcczztqkVo(List<HouseRentOutIncomeStatPo> rentOutIncomeStatPoList) {
        this.czsrTable = new ArrayList<>();
        this.czqfTable = new ArrayList<>();
        List<String> unitList1 = Stream.of("石家庄", "唐山", "秦皇岛", "邯郸", "邢台", "保定", "张家口", "承德",
                "廊坊", "沧州", "衡水", "雄安", "省机动局", "省本部", "全省").toList();
        if (rentOutIncomeStatPoList.size() == 1) {
            this.zsr = HouseReportVo.PLACEHOLDER;
            this.czsrtbzj = HouseReportVo.PLACEHOLDER;
            this.yswcl = HouseReportVo.PLACEHOLDER;
            this.xsjdcj = HouseReportVo.PLACEHOLDER + "于序时进度" + HouseReportVo.PLACEHOLDER;
            this.czsrtbzzUnitCount = HouseReportVo.PLACEHOLDER;
            this.czsrzjMaxUnit = HouseReportVo.PLACEHOLDER + "公司出租收入增加最多，";
            this.czsrtbxjUnitCount = HouseReportVo.PLACEHOLDER;
            this.czsrtbxjjdUnit = "，其中" + HouseReportVo.PLACEHOLDER + "下降幅度较大";
            this.czsrdyxsjdtbxjjdUnit = HouseReportVo.PLACEHOLDER;
            for (String unit : unitList1) {
                this.czsrTable.add(new HouseCzsrTableVo(unit));
            }
        } else {
            for (String unit : unitList1) {
                HouseCzsrTableVo t = new HouseCzsrTableVo(unit);
                this.czsrTable.add(t);
                for (HouseRentOutIncomeStatPo x : rentOutIncomeStatPoList) {
                    if ("全省".equals(x.getAreaName())) {
                        DecimalFormat numberFormat = new DecimalFormat("#,##0.0");
                        DecimalFormat integerFormat = new DecimalFormat("#,###");
                        DecimalFormat percentFormat = new DecimalFormat("0.0%");
                        this.zsr = integerFormat.format(x.getIncomeAll());
                        this.czsrtbzj = (x.getYoy().compareTo(BigDecimal.ZERO) >= 0 ? "增长" : "减少")
                                + numberFormat.format(x.getYoy().abs());
                        this.yswcl = percentFormat.format(x.getBudgetFinishRate());
                        this.xsjdcj = (x.getSequentialProgressDiff().compareTo(BigDecimal.ZERO) >= 0 ? "高" : "低")
                                + "于序时进度"
                                + x.getSequentialProgressDiff().multiply(new BigDecimal("100"))
                                .setScale(1, RoundingMode.HALF_UP).abs();
                    }
                    if (unit.equals(x.getAreaName())) {
                        t.update(x);
                        break;
                    }
                }
            }
            List<HouseRentOutIncomeStatPo> czsrtbzzList = rentOutIncomeStatPoList.stream()
                    .filter(t -> !"全省".equals(t.getAreaName()) && t.getYoyPercent().compareTo(BigDecimal.ZERO) > 0)
                    .sorted(Comparator.comparing(HouseRentOutIncomeStatPo::getYoyPercent).reversed()).toList();
            this.czsrtbzzUnitCount = String.valueOf(czsrtbzzList.size());
            if (CollectionUtils.isEmpty(czsrtbzzList)) {
                this.czsrzjMaxUnit = "";
            } else {
                this.czsrzjMaxUnit = czsrtbzzList.get(0).getAreaName() +  "公司出租收入增加最多，";
            }
            List<HouseRentOutIncomeStatPo> czsrtbxjList = rentOutIncomeStatPoList.stream()
                    .filter(t -> !"全省".equals(t.getAreaName()) && t.getYoyPercent().compareTo(BigDecimal.ZERO) < 0)
                    .sorted(Comparator.comparing(HouseRentOutIncomeStatPo::getYoyPercent)).toList();
            this.czsrtbxjUnitCount = String.valueOf(czsrtbxjList.size());
            if (CollectionUtils.isEmpty(czsrtbxjList)) {
                this.czsrtbxjjdUnit = "";
            } else if (czsrtbxjList.size() == 1) {
                this.czsrtbxjjdUnit = "，其中" + czsrtbxjList.get(0).getAreaName() + "下降幅度较大";
            } else {
                BigDecimal median;
                List<BigDecimal> l = czsrtbxjList.stream().map(HouseRentOutIncomeStatPo::getYoyPercent).sorted().toList();
                int size = l.size();
                if (size % 2 == 0) {
                    int midIndex = size / 2;
                    BigDecimal firstMidValue = l.get(midIndex - 1);
                    BigDecimal secondMidValue = l.get(midIndex);
                    median = firstMidValue.add(secondMidValue).divide(new BigDecimal("2"), RoundingMode.HALF_UP);
                } else {
                    int midIndex = size / 2;
                    median = l.get(midIndex);
                }
                this.czsrtbxjjdUnit = "，其中"
                        + czsrtbxjList.stream().filter(t -> t.getYoyPercent().compareTo(median) < 0)
                        .sorted(Comparator.comparing(HouseRentOutIncomeStatPo::getYoyPercent))
                        .map(HouseRentOutIncomeStatPo::getAreaName).collect(Collectors.joining("、"))
                        + "下降幅度较大";
            }
            List<HouseRentOutIncomeStatPo> czsrdyxsjdtbxjList = rentOutIncomeStatPoList.stream()
                    .filter(t -> !"全省".equals(t.getAreaName())
                            && t.getSequentialProgressDiff().compareTo(BigDecimal.ZERO) < 0
                            && t.getYoyPercent().compareTo(BigDecimal.ZERO) < 0)
                    .sorted(Comparator.comparing(HouseRentOutIncomeStatPo::getYoyPercent)).toList();
            if (CollectionUtils.isEmpty(czsrdyxsjdtbxjList)) {
                this.czsrdyxsjdtbxjjdUnit = "无";
            } else if (czsrtbxjList.size() == 1) {
                this.czsrdyxsjdtbxjjdUnit = czsrdyxsjdtbxjList.get(0).getAreaName();
            } else {
                BigDecimal median;
                List<BigDecimal> l = czsrdyxsjdtbxjList.stream()
                        .map(HouseRentOutIncomeStatPo::getYoyPercent).sorted().toList();
                int size = l.size();
                if (size % 2 == 0) {
                    int midIndex = size / 2;
                    BigDecimal firstMidValue = l.get(midIndex - 1);
                    BigDecimal secondMidValue = l.get(midIndex);
                    median = firstMidValue.add(secondMidValue).divide(new BigDecimal("2"), RoundingMode.HALF_UP);
                } else {
                    int midIndex = size / 2;
                    median = l.get(midIndex);
                }
                this.czsrdyxsjdtbxjjdUnit = czsrdyxsjdtbxjList.stream()
                        .filter(t -> t.getYoyPercent().compareTo(median) < 0)
                        .sorted(Comparator.comparing(HouseRentOutIncomeStatPo::getYoyPercent))
                        .map(HouseRentOutIncomeStatPo::getAreaName).collect(Collectors.joining("、"));
            }
        }
        // todo: 出租欠费
        List<String> unitList2 = Stream.of("石家庄", "唐山", "秦皇岛", "邯郸", "邢台", "保定", "张家口", "承德",
                "廊坊", "沧州", "衡水", "雄安", "省机动局", "省本部", "全省合计").toList();
        for (String unit : unitList2) {
            this.czqfTable.add(new HouseCzqfTableVo(unit));
        }
        this.czqfjdUnit = HouseReportVo.PLACEHOLDER;
        this.czsrtbzzqfjdUnit = HouseReportVo.PLACEHOLDER;
        this.czsrtbzzqfjd = "其中" + HouseReportVo.PLACEHOLDER + "累计欠费" + HouseReportVo.PLACEHOLDER + "万元，";
        this.czsrdyxsjdtbxjqf = HouseReportVo.PLACEHOLDER + "出租收入预算完成率低于序时进度，同比下降"
                + HouseReportVo.PLACEHOLDER + "，且累计欠费" + HouseReportVo.PLACEHOLDER + "万元，"
                + HouseReportVo.PLACEHOLDER + "公司应切实加强出租管理。";
    }
}
