package com.nokia.finance.tasks.pojo.vo.house_report;

import com.nokia.finance.tasks.pojo.po.house_report.HouseBuildingStatPo;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Data
@NoArgsConstructor
public class HouseZyztqkVo {
    /**
     * 全省共有房产建筑物
     */
    private String fcsl;
    /**
     * 建筑面积
     */
    private String zmj;
    /**
     * 原值
     */
    private String zcyz;
    /**
     * 核心商圈建筑面积
     */
    private String hxsq;
    /**
     * 一般商圈
     */
    private String ybsq;
    /**
     * 全省房屋数量及建筑面积排名前5位的单位
     */
    private String jzmjTopUnit;
    /**
     * 房屋建筑面积均超
     */
    private String jzmjTopMin;
    /**
     * 合计占全省房屋建筑面积
     */
    private String jzmjTopPercent;
    /**
     * 统计表格
     */
    private List<HouseZyztqkTableVo> table;

    public HouseZyztqkVo(List<HouseBuildingStatPo> list) {
        this.table = new ArrayList<>();
        DecimalFormat integerFormat = new DecimalFormat("#,###");
        DecimalFormat numberFormat = new DecimalFormat("#,##0.0");
        DecimalFormat percentFormat = new DecimalFormat("0%");
        BigDecimal jzmj = BigDecimal.ZERO;
        for (HouseBuildingStatPo t : list) {
            this.table.add(new HouseZyztqkTableVo(t));
            if ("合计".equals(t.getAreaName())) {
                this.fcsl = integerFormat.format(t.getFcsl());
                this.zmj = numberFormat.format(t.getZmj().divide(new BigDecimal("10000"), RoundingMode.HALF_UP));
                this.zcyz = integerFormat.format(t.getZcyz().divide(new BigDecimal("10000"), RoundingMode.HALF_UP));
                this.hxsq = integerFormat.format(t.getHxsq().divide(new BigDecimal("10000"), RoundingMode.HALF_UP));
                this.ybsq = integerFormat.format(t.getYbsq().divide(new BigDecimal("10000"), RoundingMode.HALF_UP));
                jzmj = t.getZmj();
            }
        }
        if (jzmj.compareTo(BigDecimal.ZERO) == 0) {
            this.fcsl = HouseReportVo.PLACEHOLDER;
            this.zmj = HouseReportVo.PLACEHOLDER;
            this.zcyz = HouseReportVo.PLACEHOLDER;
            this.hxsq = HouseReportVo.PLACEHOLDER;
            this.ybsq = HouseReportVo.PLACEHOLDER;
            this.jzmjTopUnit = HouseReportVo.PLACEHOLDER;
            this.jzmjTopMin = HouseReportVo.PLACEHOLDER;
            this.jzmjTopPercent = HouseReportVo.PLACEHOLDER;
            this.table = new ArrayList<>();
            for (HouseBuildingStatPo t : list) {
                this.table.add(new HouseZyztqkTableVo(t.getAreaName()));
            }
            return;
        }
        List<HouseBuildingStatPo> jzmjTopList = list.stream()
                .filter(t -> !"合计".equals(t.getAreaName()))
                .sorted(Comparator.comparing(HouseBuildingStatPo::getZmj).reversed()).limit(5).toList();
        this.jzmjTopUnit = jzmjTopList.stream()
                .map(HouseBuildingStatPo::getAreaName).collect(Collectors.joining("、"));
        this.jzmjTopMin = integerFormat.format(jzmjTopList.get(4).getZmj()
                .divide(new BigDecimal("10000"), RoundingMode.HALF_UP));
        BigDecimal jzmjTopSum = jzmjTopList.stream()
                .map(HouseBuildingStatPo::getZmj).reduce(BigDecimal.ZERO, BigDecimal::add);
        this.jzmjTopPercent = percentFormat.format(jzmjTopSum.divide(jzmj, RoundingMode.HALF_UP));
    }
}
