package com.nokia.finance.tasks.car;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.dao.car.CarGongCheShouFeiDao;
import com.nokia.finance.tasks.pojo.po.common.AreaPo;
import com.nokia.finance.tasks.pojo.po.common.OrganizationPo;
import com.nokia.finance.tasks.service.car.CarService;
import com.nokia.finance.tasks.service.common.AreaService;
import com.nokia.finance.tasks.service.common.OrganizationService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.util.CollectionUtils;

import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Stream;

@Slf4j
@SpringBootTest
@ActiveProfiles("dev")
class CarGongCheShiYongTests3 {
    @Autowired
    CarService carService;
    @Autowired
    OrganizationService organizationService;
    @Autowired
    AreaService areaService;
    @Autowired
    CarGongCheShouFeiDao carGongCheShouFeiDao;

    /**
     * 执行任务
     */
    @Test
    void runJob() {
        // 数据目录
        Path dir = Paths.get("test/data/yctjfxmx");
        try (Stream<Path> stream = Files.list(dir)) {
            // 获取数据目录下的文件列表
            List<Path> pathList = stream.filter(t -> t.toString().endsWith(".xlsx")).sorted().toList();
            log.info("数据文件列表: {}", pathList);
            if (CollectionUtils.isEmpty(pathList)) {
                throw new MyRuntimeException("没有文件");
            }
            for (Path path : pathList) {
                singleJob(path);
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
        }
    }

    /**
     * 处理单个文件
     *
     * @param path 文件路径
     */
    public void singleJob(Path path) throws Exception {
        List<Map<String, String>> list = readFile(path);
        List<Map<String, String>> distinctList = dataProcessing(path, list);
        Path csvPath = toCsv(path, distinctList);
    }

    /**
     * 读取文件
     *
     * @param path 文件路径
     */
    public List<Map<String, String>> readFile(Path path) throws Exception {
        log.info("读取: {}", path);
        List<String> headers = Stream.of("shen_qing_dan_hao", "pai_qian_dan_hao", "shen_qing_lei_xing",
                "che_pai_hao", "che_liang_lei_xing", "che_liang_ji_bie", "si_ji_ming_cheng", "yong_che_ren",
                "yong_che_bu_men", "yong_che_shi_you", "chu_fa_di", "mu_di_di", "chu_fa_shi_jian", "gui_dui_shi_jian",
                "xing_shi_li_cheng", "chu_che_shi_chang").toList();
        try (InputStream inputStream = Files.newInputStream(path);
             Workbook workbook = new XSSFWorkbook(inputStream)
        ) {
            List<Map<String, String>> resultList = new ArrayList<>();
            // 读取第一个工作表
            Sheet sheet = workbook.getSheetAt(0);
            // 表头行
            Row headerRow = sheet.getRow(0);
            // 列数
            int columnCount = headerRow.getPhysicalNumberOfCells();
            log.info("columnCount: {}", columnCount);
            // 最后行数
            int lastRowNum = sheet.getLastRowNum();
            log.info("lastRowNum: {}", lastRowNum);
            if (lastRowNum == 0) {
                throw new MyRuntimeException(path.getFileName() + " 数据0条");
            }
            // 遍历行
            for (int i = 1; i <= lastRowNum; i++) {
                Row row = sheet.getRow(i);
                if (row == null) {
                    continue;
                }
                Map<String, String> rowMap = new LinkedHashMap<>();
                // 遍历列
                for (int j = 0; j < columnCount; j++) {
                    String header = headers.get(j);
                    String cellValue = "";
                    rowMap.put(header, cellValue);
                    Cell cell = row.getCell(j);
                    if (cell == null) {
                        continue;
                    }
                    switch (cell.getCellType()) {
                        case STRING:
                            boolean skipTrim = "chu_fa_shi_jian".equals(header)
                                    || "gui_dui_shi_jian".equals(header);
                            // 删除字符串空白字符
                            cellValue = skipTrim ? cell.getStringCellValue()
                                    : org.springframework.util.StringUtils.trimAllWhitespace(cell.getStringCellValue());
                            break;
                        case NUMERIC:
                            if (DateUtil.isCellDateFormatted(cell)) {
                                cellValue = DateUtil.getLocalDateTime(cell.getNumericCellValue())
                                        .format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                                break;
                            }
                            cellValue = String.valueOf(cell.getNumericCellValue());
                            break;
                        case BOOLEAN:
                            cellValue = String.valueOf(cell.getBooleanCellValue());
                            break;
                        default:
                            break;
                    }
                    rowMap.put(headers.get(j), cellValue);
                }
                filter1(resultList, rowMap);
            }
            return resultList;
        }
    }

    public void filter1(List<Map<String, String>> list, Map<String, String> map) {
        String yongCheShiYou = Objects.requireNonNullElse(map.get("yong_che_shi_you"), "");
        String muDiDi = Objects.requireNonNullElse(map.get("mu_di_di"), "");
        boolean flag = (StringUtils.containsAny(yongCheShiYou, "出差", "开会", "会议", "接站",
                "培训", "送站", "送机", "接站", "接人", "机场", "站")
                || StringUtils.containsAny(muDiDi, "出差", "开会", "会议", "接站", "培训", "送站",
                "送机", "接站", "接人", "机场", "站"))
                && !StringUtils.containsAny(yongCheShiYou, "障", "营销", "查", "物料", "省公司",
                "省分", "集团", "勘察", "调研", "审计", "巡", "调测", "维护", "修", "处理", "设备", "客户", "交流", "办事",
                "访", "政府", "市委", "委员会", "县委", "基站", "收费站", "公交站", "加油站", "服务站", "检查站", "街", "路",
                "检测站", "液化气站", "总部", "省委", "党建", "机房", "调", "安装", "领导")
                && !StringUtils.containsAny(muDiDi, "障", "营销", "查", "物料", "省公司",
                "省分", "集团", "勘察", "调研", "审计", "巡", "调测", "维护", "修", "处理", "设备", "客户", "交流", "办事",
                "访", "政府", "市委", "委员会", "县委", "基站", "收费站", "公交站", "加油站", "服务站", "检查站", "街", "路",
                "检测站", "液化气站", "总部", "省委", "党建", "机房", "调", "安装", "领导");
        if (flag) {
            list.add(map);
        }
    }

    /**
     * 数据加工
     *
     * @param path 文件路径
     * @param list 数据
     */
    public List<Map<String, String>> dataProcessing(Path path, List<Map<String, String>> list) {
        Integer yearMonth = 202402;
        List<Map<String, String>> staffs = carGongCheShouFeiDao.getStaffs(yearMonth);
        Set<String> specialCar = carGongCheShouFeiDao.getSpecialCar();
        List<OrganizationPo> secondOrgs = organizationService.getSecondOrgs();
        List<OrganizationPo> thirdOrgs = organizationService.getThirdOrgs();
        Map<String, OrganizationPo> orgMap = organizationService.getOrgMap(secondOrgs, thirdOrgs);
        Map<String, List<OrganizationPo>> thirdOrganizationListMap =
                organizationService.getThirdOrganizationListMap(secondOrgs, thirdOrgs);
        List<AreaPo> cities = areaService.getCities();
        List<AreaPo> districts = areaService.getDistricts();
        Map<String, AreaPo> areaMap = areaService.getAreaMap(cities, districts);
        Map<String, List<AreaPo>> districtListMap = areaService.getDistrictListMap(cities, districts);
        List<Map<String, String>> resultList = new ArrayList<>();
        for (Map<String, String> map : list) {
//            String shenQingYongCheShiJian = map.get("shen_qing_yong_che_shi_jian");
//            LocalDateTime localDateTime = LocalDateTime.parse(shenQingYongCheShiJian, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
//            String yearMonth = localDateTime.format(DateTimeFormatter.ofPattern("yyyyMM"));
//            String year = String.valueOf(localDateTime.getYear());
//            String month = String.valueOf(localDateTime.getMonthValue());
//            map.put("year_month", yearMonth);
//            map.put("year_no", year);
//            map.put("month_no", month);
            String chuCheShiChang = map.get("chu_che_shi_chang");
            String chuCheShiChangSeconds = String.valueOf(durationStrToSeconds(chuCheShiChang));
            map.put("chu_che_shi_chang_seconds", chuCheShiChangSeconds);
            String rawChePaiHao = map.get("che_pai_hao");
//            map.put("raw_che_pai_hao", rawChePaiHao);
            String chePaiHao = carService.getChePai(rawChePaiHao);
            map.put("che_pai_hao", chePaiHao);
//            String chePaiFail = carService.chePaiFail(rawChePaiHao);
//            map.put("che_pai_fail", chePaiFail);
            String yongCheBuMen = map.get("yong_che_bu_men");
            String firstUnit = carService.getFirstUnit(yongCheBuMen);
            map.put("first_unit", firstUnit);
            String secondUnit = carService.getSecondUnit(yongCheBuMen, firstUnit);
            map.put("second_unit", secondUnit);
            String thirdUnit = carService.getThirdUnit(yongCheBuMen, secondUnit);
            map.put("third_unit", thirdUnit);
//            String areaNo = carService.getAreaNo(secondOrgs, yongCheDanWei);
//            map.put("area_no", areaNo);
//            String areaName = carService.getOrgName(orgMap, areaNo);
//            map.put("area_name", areaName);
//            String cityNo = carService.getCityNo(thirdOrganizationListMap, areaNo, areaName, yongCheDanWei);
//            map.put("city_no", cityNo);
//            String cityName = carService.getOrgName(orgMap, cityNo);
//            map.put("city_name", cityName);
//            String areaNo2 = carService.getAreaNo2(areaName, cityName);
//            map.put("area_no2", areaNo2);
//            String areaName2 = carService.getOrgName(orgMap, areaNo2);
//            map.put("area_name2", areaName2);
//            String cityId = carService.getCityId(cities, yongCheDanWei);
//            map.put("city_id", cityId);
//            String city = carService.getAreaName(areaMap, cityId);
//            map.put("city", city);
//            String districtId = carService.getDistrictId(districtListMap, cityId, cityName, yongCheDanWei);
//            map.put("district_id", districtId);
//            String district = carService.getAreaName(areaMap, districtId);
//            map.put("district", district);
//            map.put("source", path.getFileName().toString());
            filter2(resultList, map, specialCar, staffs);
        }
        return resultList;
    }

    public void filter2(List<Map<String, String>> list, Map<String, String> map, Set<String> specialCar,
                        List<Map<String, String>> staffs) {
//        String xingShiLiCheng = Objects.requireNonNullElse(map.get("xing_shi_li_cheng"), "");
//        String chuCheShiChangSeconds = Objects.requireNonNullElse(map.get("chu_che_shi_chang_seconds"), "");
        String yongCheShiYou = Objects.requireNonNullElse(map.get("yong_che_shi_you"), "");
        String yongCheRen = Objects.requireNonNullElse(map.get("yong_che_ren"), "");
        String chePaiHao = Objects.requireNonNullElse(map.get("che_pai_hao"), "");
        String firstUnit1 = Objects.requireNonNullElse(map.get("first_unit"), "");
        boolean notInStaffs = true;
        for (Map<String, String> staff : staffs) {
            String firstUnit2 = staff.get("first_unit");
            String xingMing = staff.get("xing_ming");
            // 用车人或用车事由包含交通补贴人员且单位相同
            if (firstUnit1.equals(firstUnit2)
                    && (yongCheRen.replaceAll("[^一-龥]", "").equals(xingMing)
                    || yongCheShiYou.contains(xingMing))) {
                notInStaffs = false;
                break;
            }
        }
        if (notInStaffs) {
            return;
        }
        if (specialCar.contains(chePaiHao)) {
            log.info("特殊车辆: {}", chePaiHao);
            return;
        }
//        if (new BigDecimal(chuCheShiChangSeconds).compareTo(new BigDecimal("600")) < 0
//                || new BigDecimal(xingShiLiCheng).compareTo(BigDecimal.ZERO) = 0
//                || new BigDecimal(xingShiLiCheng).compareTo(new BigDecimal("20")) >= 0) {
//            return;
//        }
        list.add(map);
    }

    /**
     * 生成csv
     *
     * @param path 源文件路径
     * @param list 数据
     */
    public Path toCsv(Path path, List<Map<String, String>> list) throws Exception {
        log.info("条数：{}", list.size());
        Files.createDirectories(Paths.get("test/history/"));
        Path csvPath = Paths.get("test/history/" + path.getFileName() + ".csv");
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(csvPath),
                StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            // 添加bom头避免excel乱码
            osw.write('\ufeff');
//            Map<String, String> header = list.get(0);
//            // 表头
//            printer.printRecord(header.keySet());
            printer.printRecord("申请单号", "派遣单号", "申请类型", "车牌号", "车辆类型", "车辆级别", "司机名称", "用车人", "用车部门", "用车事由", "出发地", "目的地", "出发时间", "归队时间", "行驶里程", "出车时长", "出车时长秒", "用车单位一级", "用车单位二级", "用车单位三级");
            for (Map<String, String> map : list) {
                printer.printRecord(
                        map.get("shen_qing_dan_hao"),
                        map.get("pai_qian_dan_hao"),
                        map.get("shen_qing_lei_xing"),
                        map.get("che_pai_hao"),
                        map.get("che_liang_lei_xing"),
                        map.get("che_liang_ji_bie"),
                        map.get("si_ji_ming_cheng"),
                        map.get("yong_che_ren"),
                        map.get("yong_che_bu_men"),
                        map.get("yong_che_shi_you"),
                        map.get("chu_fa_di"),
                        map.get("mu_di_di"),
                        map.get("chu_fa_shi_jian"),
                        map.get("gui_dui_shi_jian"),
                        map.get("xing_shi_li_cheng"),
                        map.get("chu_che_shi_chang"),
                        map.get("chu_che_shi_chang_seconds"),
                        map.get("first_unit"),
                        map.get("second_unit"),
                        map.get("third_unit")
                );
            }
        }
        return csvPath;
    }

    /**
     * 时长字符串转秒数
     * @param durationStr 时长字符串
     */
    public long durationStrToSeconds(String durationStr) {
        if (StringUtils.isEmpty(durationStr) || "0".equals(durationStr)) {
            return 0;
        }
        // 定义正则表达式，匹配时分秒、分钟秒和秒三种格式
        String regex = "(\\d+时)?(\\d+分)?(\\d+秒)";

        // 使用Pattern和Matcher进行匹配
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(durationStr);

        // 检查是否匹配成功
        if (!matcher.matches()) {
            return 0;
        }

        // 提取时、分、秒的数值
        long hours = 0;
        long minutes = 0;
        long seconds;

        // 根据匹配到的组数确定时、分、秒的值
        if (matcher.group(1) != null) {
            hours = Long.parseLong(matcher.group(1).substring(0, matcher.group(1).length() - 1));
        }
        if (matcher.group(2) != null) {
            minutes = Long.parseLong(matcher.group(2).substring(0, matcher.group(2).length() - 1));
        }
        seconds = Long.parseLong(matcher.group(3).substring(0, matcher.group(3).length() - 1));

        // 将时、分、秒转换为总秒数
        return hours * 3600L + minutes * 60L + seconds;
    }
}
